import React, { useEffect, useState } from 'react';
import { View, Text, Switch, Platform, Linking } from 'react-native';
import * as Notifications from 'expo-notifications';

import { Header, PageWrapper, MenuButton, WarningModal } from '../../../../components';
import { NAVIGATION_PAGES } from '../../../../types';
import { Colors } from '../../../../theme';

import UserPenIcon from '../../../../../assets/icons/user-pen.svg';
import UserPlusIcon from '../../../../../assets/icons/user-plus.svg';
import BellIcon from '../../../../../assets/icons/bell.svg';
import MailIcon from '../../../../../assets/icons/mail.svg';
import FAQIcon from '../../../../../assets/icons/faq.svg';
import DocumentIcon from '../../../../../assets/icons/document.svg';
import ShieldIcon from '../../../../../assets/icons/shield.svg';
import ExitIcon from '../../../../../assets/icons/exit.svg';
import UserXMark from '../../../../../assets/icons/user-xmark.svg';

import type { MenuButtonType } from '../../../../types/components';
import { StoreType, storage } from 'src/storage';
import { CommonActions } from '@react-navigation/native';

const buttons: MenuButtonType[] = [
  {
    label: 'Edit Profile',
    icon: <UserPenIcon fill={Colors.DARK_BLUE} width={20} height={20} />,
    buttonFn: (navigation) => {
      navigation.navigate(NAVIGATION_PAGES.EDIT_PERSONAL_INFO);
    }
  },
  {
    label: 'Invite a Friend',
    icon: <UserPlusIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'Notification',
    icon: <BellIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'Contact Us',
    icon: <MailIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'FAQs',
    icon: <FAQIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'Terms & Conditions',
    icon: <DocumentIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'Privacy Notice',
    icon: <ShieldIcon fill={Colors.DARK_BLUE} width={20} height={20} />
  },
  {
    label: 'Logout',
    icon: <ExitIcon fill={Colors.RED} width={20} height={20} />,
    red: true,
    buttonFn: (navigation) => {
      storage.remove('token');
      storage.remove('uid');

      navigation.dispatch(
        CommonActions.reset({
          index: 1,
          routes: [{ name: NAVIGATION_PAGES.WELCOME }]
        })
      );
    }
  },
  {
    label: 'Delete account',
    icon: <UserXMark fill={Colors.RED} width={20} height={20} />,
    red: true
  }
];

const Settings = () => {
  const [isSubscribed, setIsSubscribed] = useState(false);
  const [warningVisible, setWarningVisible] = useState(false);
  const [askPermission, setAskPermission] = useState(false);
  const [shouldOpenWarningModal, setShouldOpenWarningModal] = useState(false);

  useEffect(() => {
    const subscribed = (storage.get('subscribed', StoreType.BOOLEAN) as boolean) ?? false;
    setIsSubscribed(subscribed);
  }, []);

  const handleSubscribe = async () => {
    const token = await registerForPushNotificationsAsync();
    if (token) {
      console.log(token);
      storage.set('subscribed', true);
      setIsSubscribed(true);

      Notifications.addNotificationReceivedListener((notification) => {
        console.log('notification', notification);
      });

      Notifications.addNotificationResponseReceivedListener((response) => {
        const data = response.notification.request.content.data;
        console.log('data', data);
      });
    }
  };

  const toggleSwitch = async () => {
    if (isSubscribed) {
      storage.set('subscribed', false);
      setIsSubscribed(false);
    } else {
      setAskPermission(true);
    }
  };

  async function registerForPushNotificationsAsync() {
    let token;
    const { status: existingStatus } = await Notifications.getPermissionsAsync();
    let finalStatus = existingStatus;
    if (existingStatus !== 'granted') {
      const { status } = await Notifications.requestPermissionsAsync();
      finalStatus = status;
    }
    if (finalStatus !== 'granted') {
      setWarningVisible(true);
      return null;
    }
    token = (await Notifications.getExpoPushTokenAsync()).data;

    if (Platform.OS === 'android') {
      Notifications.setNotificationChannelAsync('default', {
        name: 'default',
        importance: Notifications.AndroidImportance.MAX,
        vibrationPattern: [0, 250, 250, 250],
        lightColor: '#FF231F7C'
      });
    }

    return token;
  }

  return (
    <PageWrapper>
      <Header label={'Settings'} />
      {buttons.map((button, index) => (
        <MenuButton
          key={index}
          label={button.label}
          icon={button.icon}
          red={button.red}
          buttonFn={button.buttonFn}
        />
      ))}
      <View
        style={{
          display: 'flex',
          flexDirection: 'row',
          justifyContent: 'space-between',
          marginTop: 20
        }}
      >
        <Text style={{ color: Colors.DARK_BLUE, fontSize: 16, fontWeight: 'bold' }}>
          Notifications
        </Text>
        <Switch
          trackColor={{ false: Colors.LIGHT_GRAY, true: Colors.DARK_BLUE }}
          thumbColor={Colors.WHITE}
          onValueChange={toggleSwitch}
          value={isSubscribed}
        />
      </View>
      <WarningModal
        isVisible={askPermission}
        onClose={() => setAskPermission(false)}
        onModalHide={() => {
          if (shouldOpenWarningModal) {
            setShouldOpenWarningModal(false);
            handleSubscribe();
          }
        }}
        type="success"
        action={() => {
          setAskPermission(false);
          setShouldOpenWarningModal(true);
        }}
        message="To use this feature we need your permission to access your notifications. If you press OK your system will ask you to confirm permission to receive notifications from NomadMania."
      />
      <WarningModal
        isVisible={warningVisible}
        onClose={() => setWarningVisible(false)}
        type="success"
        action={() => {
          Platform.OS === 'ios' ? Linking.openURL('app-settings:') : Linking.openSettings();
          setWarningVisible(false);
        }}
        message="NomadMania app needs notification permissions to function properly. Open settings?"
      />
    </PageWrapper>
  );
};

export default Settings;
